@@ -9,7 +9,7 @@ PDL::IO::FlexRaw -- A flexible binary i/o format for PerlDL.
 
         ($x,$y,...) = readflex("filename" [, $hdr])
         ($x,$y,...) = mapflex("filename" [, $hdr] [, $opts])
-        
+
         $hdr = writeflex($file, $pdl1, $pdl2,...)
         writeflexhdr($file, $hdr)
 
@@ -43,7 +43,7 @@ the header file (`banana.hdr') could look like
 
 The data can then be input using
 
-	$a = readflex('banana')[1];
+	$a = (readflex('banana'))[1];
 
 The format of the hdr file is an extension of that used by FastRaw.
 Comment lines (starting with #) are allowed, as are descriptive names
@@ -81,13 +81,14 @@ swapped.
 The optional C<$hdr> argument allows the use of an anonymous array to
 give header information, rather than using a .hdr file.  For example,
 
-	$header = [ 
+	$header = [
 	    {Type => 'f77'},
-	    {Type => 'float', NDims => 3, Dims => [ 4,600,600 ] } 
+	    {Type => 'float', NDims => 3, Dims => [ 4,600,600 ] }
 	];
 	@a = readflex('banana',$header);
 
-reads our example file again.
+reads our example file again.  As a special case, when NDims is 1, Dims
+may be given as a scalar.
 
 Within PDL, readflex and writeflex can be used to write several pdls
 to a single file -- e.g.
@@ -110,6 +111,31 @@ The reading of compressed data is switched on automatically if the
 filename requested ends in .gz or .Z, or if the originally specified
 filename does not exist, but one of these compressed forms does.
 
+If writeflex and readflex are given a reference to a file handle as a
+first parameter instead of a filename, then the data is read or
+written to the open filehandle. This gives an easy way to read an
+arbitrary slice in a big data volume, as in the following example:
+
+	use PDL;
+	use PDL::IO::FastRaw;
+
+        open(DATA, "raw3d.dat");
+        binmode(DATA);
+
+        # assume we know the data size from an external source
+        ($width, $height, $data_size) = (256,256, 4);
+
+        my $slice_num = 64;   # slice to look at
+        # Seek to slice
+        seek(DATA, $width*$height*$data_size * $slice_num, 0);
+        $pdl = readflex \*DATA, [{Dims=>[$width, $height], Type=>'long'}];
+
+WARNING: In later versions of perl (5.8 and up) you must be sure that your file
+is in "raw" mode (see the perlfunc man page entry for "binmode", for
+details).  Both readflex and writeflex automagically switch the file
+to raw mode for you -- but in code like the snipped above, you could
+end up seeking the wrong byte if you forget to make the binmode() call.
+
 Mapflex memory maps, rather than reads, the data files.  Its interface
 is similar to `readflex'.  Extra options specify if the data is to be
 loaded `ReadOnly', if the data file is to be `Creat'-ed anew on the
@@ -156,7 +182,8 @@ Read a binary file with flexible format specification
 =for usage
 
  ($x,$y,...) = readflex("filename" [, $hdr])
-   
+ ($x,$y,...) = readflex(FILEHANDLE [, $hdr])
+
 
 =head2 writeflex
 
@@ -167,10 +194,11 @@ Write a binary file with flexible format specification
 =for usage
 
   $hdr = writeflex($file, $pdl1, $pdl2,...)
-  
+  $hdr = writeflex(FILEHANDLE, $pdl1, $pdl2,...)
+
 
 =head2 mapflex
- 
+
 =for ref
 
 Memory map a binary file with flexible format specification
@@ -183,11 +211,11 @@ Memory map a binary file with flexible format specification
 
 =head1 AUTHOR
 
-Copyright (C) Robin Williams <rjrw@ast.leeds.ac.uk> 1997.  
+Copyright (C) Robin Williams <rjrw@ast.leeds.ac.uk> 1997.
 All rights reserved. There is no warranty. You are allowed
 to redistribute this software / documentation under certain
-conditions. For details, see the file COPYING in the PDL 
-distribution. If this file is separated from the PDL distribution, 
+conditions. For details, see the file COPYING in the PDL
+distribution. If this file is separated from the PDL distribution,
 the copyright notice should be included in the file.
 
 
@@ -198,7 +226,7 @@ package PDL::IO::FlexRaw;
 use PDL;
 use Exporter;
 use FileHandle;
-use PDL::Types;
+use PDL::Types ':All';
 use PDL::IO::Misc qw(bswap2 bswap4 bswap8);
 
 @PDL::IO::FlexRaw::ISA = qw/Exporter/;
@@ -206,24 +234,44 @@ use PDL::IO::Misc qw(bswap2 bswap4 bswap8);
 @EXPORT = qw/writeflex writeflexhdr readflex mapflex/;
 
 # Cast type numbers in concrete, for external file's sake...
-%flexnames = (
-   $PDL_B => 'byte', $PDL_S => 'short',
-   $PDL_US => 'ushort', $PDL_L => 'long',
-   $PDL_F => 'float', $PDL_D => 'double');
+%flexnames = ( map {(typefld($_,'numval') => typefld($_,'ioname'))}
+	       typesrtkeys());
+%flextypes = ( map {(typefld($_,'ioname') => typefld($_,'numval'),
+		     typefld($_,'numval') => typefld($_,'numval'),
+		     lc typefld($_,'ppsym') =>  typefld($_,'numval'),
+		    )}
+	       typesrtkeys());
+%flexswap = ( map {my $val = typefld($_,'numval');
+		   my $nb = PDL::Core::howbig($val);
+		   ($val =>  $nb > 1 ? "bswap$nb" : undef)}
+	      typesrtkeys());
 
-%flextypes = (
-'byte'   => $PDL_B, '0' => $PDL_B, 'b' => $PDL_B, 'c' => $PDL_B, 
-'short'  => $PDL_S, '1' => $PDL_S, 's' => $PDL_S, 
-'ushort' => $PDL_US,'2' => $PDL_US,'u' => $PDL_US,
-'long'   => $PDL_L, '3' => $PDL_L, 'l' => $PDL_L, 
-'float'  => $PDL_F, '4' => $PDL_F, 'f' => $PDL_F, 
-'double' => $PDL_D, '5' => $PDL_D, 'd' => $PDL_D
-);
+# use Data::Dumper;
+# print Dumper \%flexnames;
+# print Dumper \%flextypes;
+# print Dumper \%flexswap;
+
+# %flexnames = (
+#    $PDL_B => 'byte', $PDL_S => 'short',
+#    $PDL_US => 'ushort', $PDL_L => 'long',
+#    $PDL_F => 'float', $PDL_D => 'double');
+
+# %flextypes = (
+# 'byte'   => $PDL_B, '0' => $PDL_B, 'b' => $PDL_B, 'c' => $PDL_B,
+# 'short'  => $PDL_S, '1' => $PDL_S, 's' => $PDL_S,
+# 'ushort' => $PDL_US,'2' => $PDL_US,'u' => $PDL_US,
+# 'long'   => $PDL_L, '3' => $PDL_L, 'l' => $PDL_L,
+# 'float'  => $PDL_F, '4' => $PDL_F, 'f' => $PDL_F,
+# 'double' => $PDL_D, '5' => $PDL_D, 'd' => $PDL_D
+# );
+
+$PDL::FlexRaw::verbose = 0;
 
 sub _read_flexhdr {
     my ($hname) = @_;
     my $hfile = new FileHandle "$hname"
 	or barf "Couldn't open '$hname' for reading";
+    binmode $hfile;
     my ($newfile) = 1;
     my ($tid, @str);
     my (@ret);
@@ -248,7 +296,7 @@ TOKEN:
 			    next ITEM;
 			}
 		    }
-		    barf("Bad typename '$_' in readflex") 
+		    barf("Bad typename '$_' in readflex")
 			if (!exists($flextypes{$_}));
 		    $tid = $flextypes{$_};
 		    $newfile = 0;
@@ -285,8 +333,9 @@ TOKEN:
 
 sub readchunk {
     my ($d, $pdl, $len, $name) = @_;
-    print "Reading $len at $offset from $name\n";
-    $d->read($ {$pdl->get_dataref},$len) == $len
+    print "Reading $len at $offset from $name\n"
+      if $PDL::FlexRaw::verbose;
+    read($d, ${$pdl->get_dataref}, $len) == $len
 	or barf "Couldn't read enough data from '$name'";
     $pdl->upd_data();
     $offset += $len;
@@ -313,14 +362,21 @@ sub mapchunk {
 }
 
 sub readflex {
-    barf 'Usage ($x,$y,...) = readflex("filename" [, \@hdr])' 
+    barf 'Usage ($x,$y,...) = readflex("filename"|FILEHANDLE [, \@hdr])'
 	if $#_ > 1;
     my ($name,$h) = @_;
     my ($hdr, $pdl, $len, @out, $chunk, $chunkread, $data);
     local ($offset) = 0;
     my ($newfile, $swapbyte, $f77mode, $zipt) = (1,0,0,0);
+    my $d;
 
-    if ($name =~ s/\.gz$// || $name =~ s/\.Z$// || 
+    # Test if $name is a file handle
+    if (defined fileno($name)) {
+	$d = $name;
+	binmode($d); 
+    }
+    else {
+    if ($name =~ s/\.gz$// || $name =~ s/\.Z$// ||
 	(! -e $name && (-e $name.'.gz' || -e $name.'.Z'))) {
 	$data = "gzip -dcq $name |";
 	$zipt = 1;
@@ -329,12 +385,13 @@ sub readflex {
     }
 
     my ($size) = (stat $name)[7];
-    my ($d) = new FileHandle $data
+	$d = new FileHandle $data
 	or barf "Couldn't open '$data' for reading";
-
+    binmode $d;
     if ($#_ == 0) {
 	$h = _read_flexhdr("$name.hdr");
     }
+    }
 
 # Go through headers which reconfigure
     foreach $hdr (@$h) {
@@ -367,21 +424,23 @@ READ:
 	    }
 	}
 	if ($#_ == 1) {
-	    barf("Bad typename '$type' in readflex") 
+	    barf("Bad typename '$type' in readflex")
 		if (!defined($flextypes{$type}));
 	    $type = $flextypes{$type};
 	}
-	$pdl = PDL->zeroes ((new PDL::Type($type)), 
-			    @{$hdr->{Dims}});
-	$len = length $ {$pdl->get_dataref};    
+	$pdl = PDL->zeroes ((new PDL::Type($type)),
+			    ref $hdr->{Dims} ? @{$hdr->{Dims}} : $hdr->{Dims});
+	$len = length $ {$pdl->get_dataref};
 
 	&readchunk($d,$pdl,$len,$name) or last READ;
-	$chunkread += $len; 
+	$chunkread += $len;
 	if ($swapbyte) {
-	    bswap2($pdl) if $pdl->get_datatype == $PDL_S;
-	    bswap4($pdl) if $pdl->get_datatype == $PDL_L 
-		|| $pdl->get_datatype == $PDL_F;
-	    bswap8($pdl) if $pdl->get_datatype == $PDL_D;
+	  my $method = $flexswap{$type};
+	  $pdl->$method if $method;
+# 	    bswap2($pdl) if $pdl->get_datatype == $PDL_S;
+# 	    bswap4($pdl) if $pdl->get_datatype == $PDL_L
+# 		|| $pdl->get_datatype == $PDL_F;
+# 	    bswap8($pdl) if $pdl->get_datatype == $PDL_D;
 	}
 	if ($newfile && $f77mode) {
 	    if ($zipt || $swapbyte) {
@@ -396,7 +455,7 @@ READ:
 		    bswap4($pdl) if $swapbyte;
 		    $chunk = $pdl->copy;
 		    next SWAP if ! seek($d,$pdl->at,1);
-		    next SWAP if 
+		    next SWAP if
 			read($d,$ {$chunk->get_dataref},$len) != $len;
 		    $chunk->upd_data;
 		    bswap4($chunk) if $swapbyte;
@@ -433,9 +492,9 @@ READ:
 }
 
 sub mapflex {
-    my ($usage) 
+    my ($usage)
 	= 'Usage ($x,$y,...) = mapflex("filename" [, \@hdr] [,\%opts])';
-    my ($name) = shift @_;
+    my $name = shift;
     # reference to header array
     my ($h, $size);
     # reference to options array, with defaults
@@ -451,11 +510,11 @@ sub mapflex {
 	} elsif (ref($_) eq "HASH") {
 	    %opts = (%opts,%$_);
 	} else {
-	    barf $usage;
+	    warn $usage;
 	}
     }
 
-    if ($name =~ s/\.gz$// || $name =~ s/\.Z$// || 
+    if ($name =~ s/\.gz$// || $name =~ s/\.Z$// ||
 	(! -e $name && (-e $name.'.gz' || -e $name.'.Z'))) {
 	barf "Can't map compressed file";
     }
@@ -473,10 +532,10 @@ sub mapflex {
 	    $f77mode = 1;
 	} else {
 	    my($si) = 1;
-	    foreach (@{$hdr->{Dims}}) {
+	    foreach (ref $hdr->{Dims} ? @{$hdr->{Dims}} : $hdr->{Dims}) {
 		$si *= $_;
 	    }
-	    $size += $si * howbig ($type);
+	    $size += $si * PDL::Core::howbig ($type);
 	}
     }
 # $s now contains estimated size of data in header --
@@ -491,6 +550,8 @@ sub mapflex {
 
     $d = byte PDL->zeroes(1);
     # print "Mapping total size $size\n";
+    # use Data::Dumper;
+    # print "Options: ", Dumper(\%opts), "\n";
     $d->set_data_by_mmap($name,$size,1,($opts{ReadOnly}?0:1),
 			 ($opts{Creat}?1:0),
 			 (0644),
@@ -513,16 +574,16 @@ READ:
 	    }
 	}
 	if ($#_ == 1) {
-	    barf("Bad typename '$type' in mapflex") 
+	    barf("Bad typename '$type' in mapflex")
 		if (!defined($flextypes{$type}));
 	    $type = $flextypes{$type};
 	}
-	$pdl = PDL->zeroes ((new PDL::Type($type)), 
-			    @{$hdr->{Dims}});
-	$len = length $ {$pdl->get_dataref};    
+	$pdl = PDL->zeroes ((new PDL::Type($type)),
+			    ref $hdr->{Dims} ? @{$hdr->{Dims}} : $hdr->{Dims});
+	$len = length $ {$pdl->get_dataref};
 
 	&mapchunk($d,$pdl,$len,$name) or last READ;
-	$chunkread += $len; 
+	$chunkread += $len;
 	if ($newfile && $f77mode) {
 	    if ($opts{Creat}) {
 		$pdl->set(0,$size - 8);
@@ -555,13 +616,24 @@ READ:
 }
 
 sub writeflex {
-    barf 'Usage writeflex("filename",$pdl,...)' if $#_<0;
+    my $usage = 'Usage writeflex("filename"|FILEHANDLE,$pdl,...)';
+    barf $usage if $#_<0;
     my($name) = shift; my (@ret);
-    barf 'Usage writeflex("filename",$pdl,...)' if ref $name;    
-    my $d = new FileHandle ">$name"
+    my $d;
+
+    # Test if $name is a file handle
+    if (defined fileno($name)) {
+	$d = $name;
+	binmode $d;
+    }
+    else {
+	barf $usage if ref $name;
+	$d = new FileHandle ">$name"
 	or barf "Couldn't open '$name' for writing";
+    binmode $d;
+    }
     foreach $pdl (@_) {
-	barf 'Usage writeflex("filename",$pdl,...)' if ! ref $pdl;
+	barf $usage if ! ref $pdl;
 	# print join(' ',$pdl->getndims,$pdl->dims),"\n";
 	push @ret, {
 	    Type => $flexnames{$pdl->get_datatype},
@@ -579,7 +651,7 @@ sub writeflexhdr {
     my $hname = "$name.hdr";
     my $h = new FileHandle ">$hname"
 	or barf "Couldn't open '$hname' for writing";
-    print $h 
+    print $h
 	"# Output from PDL::IO::writeflex, data in $name\n";
     foreach (@$hdr) {
 	my ($type) = $_->{Type};
@@ -588,8 +660,9 @@ sub writeflexhdr {
 	    next;
 	}
 	print $h join("\n",$_->{Type},
-		      $_->{NDims}, 
-		      (join ' ',@{ $_->{Dims}})),"\n\n";
+		      $_->{NDims},
+		      (join ' ',ref $_->{Dims} ? @{$_->{Dims}} : $_->{Dims})),
+	"\n\n";
     }
 }
 
